package com.jianshui.invoice.task;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.jianshui.common.core.domain.entity.Companyservice;
import com.jianshui.common.core.redis.RedisCache;
import com.jianshui.common.utils.StringUtils;
import com.jianshui.common.utils.spring.SpringUtils;
import com.jianshui.framework.manager.AsyncManager;
import com.jianshui.invoice.domain.BillInfo;
import com.jianshui.invoice.domain.Invoice;
import com.jianshui.invoice.domain.InvoiceBack;
import com.jianshui.invoice.domain.dto.HXResponse;
import com.jianshui.invoice.domain.dto.QueryBillInfoDTO;
import com.jianshui.invoice.factory.IInvoiceRequestFactory;
import com.jianshui.invoice.factory.IInvoiceResponseFactory;
import com.jianshui.invoice.factory.IInvoiceServiceFactory;
import com.jianshui.invoice.mapper.InvoiceBackMapper;
import com.jianshui.invoice.mapper.InvoiceMapper;
import com.jianshui.invoice.service.IInvoiceApiService;
import com.jianshui.invoice.service.IInvoiceRequestService;
import com.jianshui.queue.utils.RedisQueueUtil;
import com.jianshui.system.service.ICompanyserviceService;
import com.jianshui.system.service.IServiceManageService;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.TimerTask;

/**
 * @Description 批量开票任务
 **/
@Slf4j
public class AddInvoiceBatchTask extends TimerTask {



    private ICompanyserviceService iCompanyserviceService;
    private IServiceManageService serviceManageService;
    private IInvoiceRequestFactory invoiceRequestFactory;
    private IInvoiceServiceFactory invoiceServiceFactory;
    private InvoiceBackMapper invoiceBackMapper;
    private String identity;
    private String order;
    
    private final String LOG_MESSAGE = "重汽批量开票接口";


    public AddInvoiceBatchTask(ICompanyserviceService iCompanyserviceService, IServiceManageService serviceManageService
            , IInvoiceRequestFactory invoiceRequestFactory,IInvoiceServiceFactory invoiceServiceFactory,InvoiceBackMapper invoiceBackMapper,String identity,String order) {
        this.iCompanyserviceService = iCompanyserviceService;
        this.serviceManageService = serviceManageService;
        this.invoiceRequestFactory = invoiceRequestFactory;
        this.invoiceServiceFactory = invoiceServiceFactory;
        this.invoiceBackMapper = invoiceBackMapper;
        this.identity = identity;
        this.order = order;
    }

    @SneakyThrows
    @Override
    public void run() {

        // 异步处理
        // 1-1 identity校验
        if (StringUtils.isEmpty(identity)) {
            log.info("{}:身份认证参数为空!",LOG_MESSAGE);
            return;
        }

        // 1-2 企业信息校验
        Companyservice companyservice = null;
        try {
            companyservice = iCompanyserviceService.selectCompanyserviceByIdentity(identity);
        } catch (Exception e) {
            log.info("{}:获取企业信息异常!identity={},e={}", LOG_MESSAGE,identity, e.toString());
            return;
        }
        if (companyservice == null) {
            log.info("{}:企业信息不存在!identity={}",LOG_MESSAGE, identity);
            return;
        }

        // 1-3 获得入口报文适配器
        JSONObject decryptResult = null;
        try {
            String requestAdapterKey = serviceManageService.getRequestAdapterKey("invoice", companyservice.getCompanyid());
            IInvoiceRequestService invoiceRequestService = invoiceRequestFactory.getService(requestAdapterKey);
            decryptResult = invoiceRequestService.decryptZq(identity,order, companyservice, "addBatch");
        } catch (Exception e) {
            log.info("{}:请求解密失败!identity={},e={}",LOG_MESSAGE, identity, e.toString());
            return;
        }

        // 2-报文处理
        cn.hutool.json.JSONObject billJson = null;
        try {
            billJson = JSONUtil.parseObj(decryptResult);
        } catch (Exception e) {
            log.info("{}:报文格式化异常!identity={}，e={}",LOG_MESSAGE, identity, e.toString());
            return;
        }
        if (billJson.get("billInfo") == null) {
            log.info("{}:未匹配到发票报文!identity={}",LOG_MESSAGE, identity);
            return;
        }

        JSONArray billJsonArray = null;
        try {
            billJsonArray = JSONUtil.parseArray(billJson.get("billInfo"));
        } catch (Exception e) {
            log.info("{}:发票报文获取异常!identity={},e={}",LOG_MESSAGE, identity, e.toString());
            return;
        }
        if (billJsonArray.isEmpty()) {
            log.info("{}:发票报文为空!identity={}",LOG_MESSAGE, identity);
            return;
        }

        // 3-报文递交开票
        for (Object billInfoObj : billJsonArray) {

            // 1-前置条件判断
            InvoiceBack invoiceBack = new InvoiceBack();
            invoiceBack.setUpdateTime(new Date());
            invoiceBack.setCreateTime(new Date());
            invoiceBack.setId(IdUtil.randomUUID());
            invoiceBack.setSystemOrderno(IdUtil.randomUUID());
            invoiceBack.setStatus("1");
            invoiceBack.setIdentity(identity);
            invoiceBack.setBackMsg("");
            invoiceBack.setBackUrl("");

            BillInfo billInfo = BeanUtil.copyProperties(billInfoObj, BillInfo.class);
            if (billInfo == null) {
                log.info("{}:报文格式错误!identity={}",LOG_MESSAGE, identity);
                continue;
            }
            billInfo.setSource("2");

            // 4-获得请求实例，并且进行扣费
            IInvoiceApiService invoiceService = null;
            try {
                String serviceKey = serviceManageService.getCompanyServiceSupplier("invoice", companyservice.getCompanyid());
                serviceManageService.companyConsume("invoice", companyservice.getCompanyid());
                billInfo.setServiceSupplierKey(serviceKey);
                invoiceService = invoiceServiceFactory.getService(serviceKey);
            } catch (Exception e) {
                log.info("{}:扣费失败!identity：{},e:{}",LOG_MESSAGE, identity, e.toString());
                continue;
            }

            // 5-开票
            try {

                try {
                    // 控制台问题，开票太快了会报9000错误，询问说添加一秒延时解决。
                    Thread.sleep(1000);
                } catch (InterruptedException e) {
                    // 处理中断异常
                    log.error("{}：延时异常：{}",LOG_MESSAGE,e.toString());
                }

//                HXResponse resultHx = invoiceService.addInvoice(billInfo, companyservice);
                HXResponse resultHx = invoiceService.addInvoiceBeatch(billInfo, companyservice);
                log.info("{}:开票identity={},返回结果result={}",LOG_MESSAGE, identity, resultHx);

                if (!resultHx.isSuccess()) {

                    QueryWrapper<InvoiceBack> queryWrapper = new QueryWrapper<>();
                    queryWrapper.eq("system_orderno", billInfo.getOutTradeOrderno());
                    InvoiceBack invoiceBackTemp = invoiceBackMapper.selectOne(queryWrapper);

                    if (invoiceBackTemp != null) {
                        invoiceBackTemp.setResultCode("9999");
                        invoiceBackTemp.setResultMsg(resultHx.getMessage());
                        invoiceBackTemp.setStatus("0");
                        invoiceBackTemp.setUpdateTime(new Date());
                        invoiceBackMapper.updateInvoiceBack(invoiceBackTemp);

                        invoiceService.callBackAisino(invoiceBackTemp, "", identity,0L);
                    } else {
                        invoiceBack.setResultCode("9999");
                        invoiceBack.setResultMsg(resultHx.getMessage());
                        invoiceBack.setSystemOrderno(billInfo.getOutTradeOrderno());
                        invoiceBack.setStatus("0");
                        invoiceBack.setCreateTime(new Date());
                        invoiceBackMapper.insertInvoiceBack(invoiceBack);

                        invoiceService.callBackAisino(invoiceBack, "", identity,0L);
                    }
                    log.info("{}:提交开票异常，开票失败!identity={},msg={}",LOG_MESSAGE, identity, resultHx.getMessage());
                }

            } catch (Exception e) {

                QueryWrapper<InvoiceBack> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("system_orderno", billInfo.getOutTradeOrderno());
                InvoiceBack invoiceBackTemp = invoiceBackMapper.selectOne(queryWrapper);

                if (invoiceBackTemp != null) {
                    invoiceBackTemp.setResultCode("9999");
                    invoiceBackTemp.setResultMsg("开票失败!");
                    invoiceBackTemp.setStatus("0");
                    invoiceBackTemp.setUpdateTime(new Date());
                    invoiceBackMapper.updateInvoiceBack(invoiceBackTemp);

                    invoiceService.callBackAisino(invoiceBackTemp, "", identity, 0L);
                } else {
                    invoiceBack.setResultCode("9999");
                    invoiceBack.setResultMsg("开票失败!");
                    invoiceBack.setSystemOrderno(billInfo.getOutTradeOrderno());
                    invoiceBack.setStatus("0");
                    invoiceBack.setCreateTime(new Date());
                    invoiceBackMapper.insertInvoiceBack(invoiceBack);

                    invoiceService.callBackAisino(invoiceBack, "",  identity,0L);
                }

                log.info("{}:提交开票异常，开票失败!identity={},e={}",LOG_MESSAGE, identity, e.toString());
            }
        }

    }
}
